import soundbox
import numpy as np
import scipy.signal as sig
import matplotlib
import matplotlib.pyplot as plt
import sys

if len(sys.argv) != 3:
    print(f"""Utilisation: {sys.argv[0]} [source] [output]

Génère le sonagramme du fichier [source] dans le fichier [output].
Passer - comme [output] fait s’afficher le sonagramme dans une fenêtre.""")
    sys.exit(1)

source_file = sys.argv[1]
output_file = sys.argv[2]

# Calcul du STFT
signal = soundbox.load_signal(source_file)

freq, time, vecs = sig.stft(signal, soundbox.samp_rate, nperseg=soundbox.samp_rate * 0.5)
values = np.absolute(vecs)

# Calcul de la fenêtre des fréquences intéressantes
high_enough = np.where(values.max(axis=1) / np.max(values) >= 0.01)
left = high_enough[0][0]
right = high_enough[0][-1]

freq = freq[left:right]
values = values[left:right]

# Génération du graphe
plt.rcParams.update({
    'figure.figsize': (10, 8),
    'font.size': 18,
    'font.family': 'Concourse T4',
})

fig, ax = plt.subplots()
ax.tick_params(axis='both', which='major', labelsize=12)
ax.pcolormesh(time, freq, values, cmap='Greys', shading='gouraud')
ax.grid(alpha=.3)

# Configuration des axes
def freq_format(value, pos):
    return f'{value:.0f} Hz'


ax.set_xlabel('Temps', labelpad=10)
ax.xaxis.set_major_formatter(matplotlib.ticker.StrMethodFormatter('{x:.0f} s'))
ax.xaxis.set_major_locator(matplotlib.ticker.MultipleLocator(1))
ax.xaxis.set_minor_formatter(matplotlib.ticker.NullFormatter())
ax.xaxis.set_minor_locator(matplotlib.ticker.MultipleLocator(.2))

ax.set_ylabel('Fréquence', labelpad=10)
ax.set_yscale('log', base=2)
ax.yaxis.set_major_locator(plt.MultipleLocator(100))
ax.yaxis.set_major_formatter(matplotlib.ticker.StrMethodFormatter('{x:.0f} Hz'))
ax.yaxis.set_minor_formatter(matplotlib.ticker.NullFormatter())
ax.yaxis.set_minor_locator(matplotlib.ticker.MultipleLocator(10))

# Rendu du résultat
if output_file == '-':
    plt.show()
else:
    plt.tight_layout()
    plt.savefig(output_file)
